package by.zerg.pokerhandsbuilder.domain;

public class Player {
	public static final int ACE = 0;
	public static final int KING = 1;
	public static final int QUEEN = 2;
	public static final int JACK = 3;
	public static final int TEN = 4;
	public static final int NINE = 5;
	public static final int EIGHT = 6;
	public static final int SEVEN = 7;
	public static final int SIX = 8;
	public static final int FIVE = 9;
	public static final int FOUR = 10;
	public static final int THREE = 11;
	public static final int TWO = 12;
	public static final int UNKNOWN = 13;

	public static final int SPADES = 0;
	public static final int HEARTS = 1;
	public static final int DIAMONDS = 2;
	public static final int CLUBS = 3;

	public static final int SB = 0;
	public static final int BB = 1;
	public static final int UTG = 2;
	public static final int UTG1 = 3;
	public static final int MP = 4;
	public static final int MP1 = 5;
	public static final int MP2 = 6;
	public static final int MP3 = 7;
	public static final int CO = 8;
	public static final int BU = 9;

	private String nickName;
	private int position;
	private float stack;
	private String cards;
	private Action action;
	private boolean isActive;

	public Player() {
		// TODO Auto-generated constructor stub
	}

	public Player(String nickName, int position, float stack, String cards,
			Action action, boolean isActive) {
		super();
		this.nickName = nickName;
		this.position = position;
		this.stack = stack;
		this.cards = cards;
		this.action = action;
		this.isActive = isActive;
	}

	public void postSmallBlind(float sb) {
		action = new Action(Action.POST_SB, sb);
		stack -= sb;
	}

	public void postBigBlind(float bb) {
		action = new Action(Action.POST_BB, bb);
		stack -= bb;
	}

	public void fold() {
		if (action != null) {
			action.setType(Action.FOLD);
		} else {
			action = new Action(Action.FOLD, 0);
		}
	}

	public void check() {
		action = new Action(Action.CHECK, 0);
	}

	public float call(float toCall) {
		if (action != null) {
			toCall -= action.getAmount();
		} else {
			action = new Action(Action.CALL, 0);
		}
		int actionType = Action.CALL;
		if (toCall >= stack) {
			toCall = stack;
			actionType = Action.ALLIN;
		}
		action = new Action(actionType, action.getAmount() + toCall);
		stack -= toCall;
		return toCall;
	}

	public float bet(float amount) {
		if (amount >= stack) {
			amount = stack;
			action = new Action(Action.ALLIN, amount);
		} else {
			action = new Action(Action.BET, amount);
		}
		stack -= amount;
		return amount;
	}

	public float raise(float amount) {
		if (action != null) {
			amount -= action.getAmount();
		} else {
			action = new Action(Action.RAISE, 0);
		}
		int actionType = Action.RAISE;
		if (amount >= stack) {
			amount = stack;
			actionType = Action.ALLIN;
		}
		action = new Action(actionType, action.getAmount() + amount);
		stack -= amount;
		return amount;
	}

	public static String getPositionName(int position) {
		switch (position) {
		case UTG:
			return "UTG";
		case UTG1:
			return "UTG+1";
		case MP:
			return "MP";
		case MP1:
			return "MP1";
		case MP2:
			return "MP2";
		case MP3:
			return "MP3";
		case CO:
			return "CO";
		case BU:
			return "BU";
		case SB:
			return "SB";
		case BB:
			return "BB";
		default:
			return null;
		}
	}

	public static int getCardNumber(String card) {
		if (card.equals("A")) {
			return ACE;
		} else if (card.equals("K")) {
			return KING;
		} else if (card.equals("Q")) {
			return QUEEN;
		} else if (card.equals("J")) {
			return JACK;
		} else if (card.equals("T")) {
			return TEN;
		} else if (card.equals("9")) {
			return NINE;
		} else if (card.equals("8")) {
			return EIGHT;
		} else if (card.equals("7")) {
			return SEVEN;
		} else if (card.equals("6")) {
			return SIX;
		} else if (card.equals("5")) {
			return FIVE;
		} else if (card.equals("4")) {
			return FOUR;
		} else if (card.equals("3")) {
			return THREE;
		} else if (card.equals("2")) {
			return TWO;
		} else {
			return UNKNOWN;
		}
	}

	public static int getCardSuit(String suit) {
		if (suit.equals("s")) {
			return SPADES;
		} else if (suit.equals("h")) {
			return HEARTS;
		} else if (suit.equals("d")) {
			return DIAMONDS;
		} else if (suit.equals("c")) {
			return CLUBS;
		} else {
			return UNKNOWN;
		}
	}

	public String getNickName() {
		return nickName;
	}

	public void setNickName(String nickName) {
		this.nickName = nickName;
	}

	public int getPosition() {
		return position;
	}

	public void setPosition(int position) {
		this.position = position;
	}

	public float getStack() {
		return stack;
	}

	public void setStack(float stack) {
		this.stack = stack;
	}

	public String getCards() {
		return cards;
	}

	public void setCards(String cards) {
		this.cards = cards;
	}

	public Action getAction() {
		return action;
	}

	public void setAction(Action action) {
		this.action = action;
	}

	public boolean isActive() {
		return isActive;
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}
}
