package by.zerg.pokerhandsbuilder.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import by.zerg.pokerhandsbuilder.domain.GameOptions;
import by.zerg.pokerhandsbuilder.domain.Player;

public class MainFrame extends JFrame {
	public static final String GO_LAST = "GO_LAST";
	public static final String GO_NEXT = "GO_NEXT";
	public static final String GO_PREV = "GO_PREV";
	public static final String GO_FIRST = "GO_FIRST";
	public static final String PLAYERS_SELECTED = "PLAYERS_SELECTED";
	public static final String SET_OPTIONS = "SET_OPTIONS";
	public static final String OPTIONS_SELECTED = "OPTIONS_SELECTED";
	public static final String FLOP_SELECTED = "FLOP_SELECTED";
	public static final String TURN_SELECTED = "TURN_SELECTED";
	public static final String RIVER_SELECTED = "RIVER_SELECTED";
	public static final String SHOWDOWN_SELECTED = "SHOWDOWN_SELECTED";
	public static final String POST_SELECTED = "POST_SELECTED";
	public static final int VIEW_OPTIONS = 0;
	public static final int VIEW_PLAYERS = 1;
	public static final int VIEW_ACTION = 2;
	public static final int VIEW_FLOP = 3;
	public static final int VIEW_TURN = 4;
	public static final int VIEW_RIVER = 5;
	public static final int VIEW_SHOWDOWN = 6;
	public static final int VIEW_POST = 7;

	private JPanel leftPanel;

	private JPanel optionsPanel;
	private JRadioButton sRadio;
	private JRadioButton fRadio;
	private JComboBox limitsList;

	private JPanel playersPanel;
	private JCheckBox[] playerActivities;
	private JTextField[] playerNames;
	private JTextField[] playerStacks;
	private JTextField[] playerCards;
	private JComboBox heroCombo;

	private JPanel wizzardPanel;
	private JPanel boardPanel;
	private JPanel flopPanel;
	private JPanel turnPanel;
	private JPanel riverPanel;
	private JTextField flop;
	private JTextField turn;
	private JTextField river;
	private JPanel showDownPanel;
	private JComboBox showDownCombo;
	private ActionPanel actionPanel;

	private ReplayerPanel replayerPanel;
	private JPanel navigatorPanel;

	private JPanel historyPanel;
	private JTextArea history;

	private JButton optionsButton;
	private JButton newButton;
	private JButton playersButton;
	private JButton flopButton;
	private JButton turnButton;
	private JButton riverButton;
	private JButton showDownButton;
	private JButton postButton;

	public MainFrame(ActionListener performer) {
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setTitle("PokerHandsBuilder v1.1 by Hitman");
		initGUI(performer);
		setVisible(true);
	}

	public void initGUI(ActionListener performer) {
		setLayout(new BorderLayout());

		leftPanel = new JPanel(new BorderLayout());
		Box leftBox = Box.createVerticalBox();

		optionsPanel = new JPanel();
		optionsPanel.setBorder(new TitledBorder("Options"));
		Box optionsBox = Box.createVerticalBox();
		JPanel gameTypePanel = new JPanel();
		gameTypePanel.add(new JLabel("Game: "));
		sRadio = new JRadioButton("SH");
		sRadio.setSelected(true);
		fRadio = new JRadioButton("FR");
		ButtonGroup radioGroup = new ButtonGroup();
		radioGroup.add(sRadio);
		radioGroup.add(fRadio);
		gameTypePanel.add(sRadio);
		gameTypePanel.add(fRadio);
		optionsBox.add(gameTypePanel);
		JPanel limitsPanel = new JPanel();
		limitsPanel.add(new JLabel("Limit: "));
		limitsList = new JComboBox(new String[] { "NL50", "NL100", "NL200" });
		limitsList.setSelectedIndex(1);
		limitsPanel.add(limitsList);
		optionsBox.add(limitsPanel);
		optionsPanel.add(optionsBox);
		leftBox.add(optionsPanel);

		playersPanel = new JPanel();
		playersPanel.setBorder(new TitledBorder("Players"));
		leftBox.add(playersPanel);

		boardPanel = new JPanel();
		boardPanel.setBorder(new TitledBorder("Board"));
		Box boardBox = Box.createVerticalBox();
		flopPanel = new JPanel();
		flopPanel.add(new JLabel("Flop: "));
		flop = new JTextField(6);
		flopPanel.add(flop);
		boardBox.add(flopPanel);
		turnPanel = new JPanel();
		turnPanel.add(new JLabel("Turn: "));
		turn = new JTextField(6);
		turnPanel.add(turn);
		boardBox.add(turnPanel);
		riverPanel = new JPanel();
		riverPanel.add(new JLabel("River: "));
		river = new JTextField(6);
		riverPanel.add(river);
		boardBox.add(riverPanel);
		boardPanel.add(boardBox);
		leftBox.add(boardPanel);

		showDownPanel = new JPanel();
		showDownPanel.setBorder(new TitledBorder("Showdown"));
		showDownPanel.add(new JLabel("Winner is: "));
		showDownCombo = new JComboBox();
		showDownPanel.add(showDownCombo);
		leftBox.add(showDownPanel);

		wizzardPanel = new JPanel();
		optionsButton = new JButton("Next");
		optionsButton.setActionCommand(OPTIONS_SELECTED);
		optionsButton.addActionListener(performer);
		wizzardPanel.add(optionsButton);
		newButton = new JButton("New");
		newButton.setActionCommand(SET_OPTIONS);
		newButton.addActionListener(performer);
		wizzardPanel.add(newButton);
		playersButton = new JButton("Next");
		playersButton.setActionCommand(PLAYERS_SELECTED);
		playersButton.addActionListener(performer);
		wizzardPanel.add(playersButton);
		flopButton = new JButton("Next");
		flopButton.setActionCommand(FLOP_SELECTED);
		flopButton.addActionListener(performer);
		wizzardPanel.add(flopButton);
		turnButton = new JButton("Next");
		turnButton.setActionCommand(TURN_SELECTED);
		turnButton.addActionListener(performer);
		wizzardPanel.add(turnButton);
		riverButton = new JButton("Next");
		riverButton.setActionCommand(RIVER_SELECTED);
		riverButton.addActionListener(performer);
		wizzardPanel.add(riverButton);
		showDownButton = new JButton("Next");
		showDownButton.setActionCommand(SHOWDOWN_SELECTED);
		showDownButton.addActionListener(performer);
		wizzardPanel.add(showDownButton);
		postButton = new JButton("Post");
		postButton.setActionCommand(POST_SELECTED);
		postButton.addActionListener(performer);
		wizzardPanel.add(postButton);
		leftBox.add(wizzardPanel);
		leftPanel.add(leftBox, BorderLayout.NORTH);

		actionPanel = new ActionPanel(performer);
		leftPanel.add(actionPanel, BorderLayout.SOUTH);
		add(leftPanel, BorderLayout.WEST);

		JPanel centralPanel = new JPanel(new BorderLayout());
		replayerPanel = new ReplayerPanel();
		centralPanel.add(replayerPanel);
		navigatorPanel = new JPanel();
		JButton firstButton = new JButton("|<");
		firstButton.setActionCommand(GO_FIRST);
		firstButton.addActionListener(performer);
		navigatorPanel.add(firstButton);
		JButton prevButton = new JButton("<<");
		prevButton.setActionCommand(GO_PREV);
		prevButton.addActionListener(performer);
		navigatorPanel.add(prevButton);
		JButton nextButton = new JButton(">>");
		nextButton.setActionCommand(GO_NEXT);
		nextButton.addActionListener(performer);
		navigatorPanel.add(nextButton);
		JButton lastButton = new JButton(">|");
		lastButton.setActionCommand(GO_LAST);
		lastButton.addActionListener(performer);
		navigatorPanel.add(lastButton);
		centralPanel.add(navigatorPanel, BorderLayout.SOUTH);
		add(centralPanel);

		history = new JTextArea(10, 0);
		historyPanel = new JPanel(new BorderLayout());
		historyPanel.add(new JScrollPane(history));
		add(historyPanel, BorderLayout.SOUTH);
	}

	public void switchView(int view) {
		switch (view) {
		case VIEW_OPTIONS:
			optionsPanel.setVisible(true);
			playersPanel.setVisible(false);
			actionPanel.setVisible(false);
			navigatorPanel.setVisible(false);
			boardPanel.setVisible(false);
			showDownPanel.setVisible(false);
			optionsButton.setVisible(true);
			newButton.setVisible(false);
			playersButton.setVisible(false);
			flopButton.setVisible(false);
			turnButton.setVisible(false);
			riverButton.setVisible(false);
			showDownButton.setVisible(false);
			postButton.setVisible(false);
			break;

		case VIEW_PLAYERS:
			optionsPanel.setVisible(false);
			playersPanel.setVisible(true);
			actionPanel.setVisible(false);
			navigatorPanel.setVisible(false);
			boardPanel.setVisible(false);
			showDownPanel.setVisible(false);
			optionsButton.setVisible(false);
			newButton.setVisible(true);
			playersButton.setVisible(true);
			flopButton.setVisible(false);
			turnButton.setVisible(false);
			riverButton.setVisible(false);
			showDownButton.setVisible(false);
			postButton.setVisible(false);
			break;

		case VIEW_ACTION:
			optionsPanel.setVisible(false);
			playersPanel.setVisible(false);
			actionPanel.setVisible(true);
			navigatorPanel.setVisible(true);
			boardPanel.setVisible(false);
			showDownPanel.setVisible(false);
			optionsButton.setVisible(false);
			newButton.setVisible(true);
			playersButton.setVisible(false);
			flopButton.setVisible(false);
			turnButton.setVisible(false);
			riverButton.setVisible(false);
			showDownButton.setVisible(false);
			postButton.setVisible(false);
			break;

		case VIEW_FLOP:
			optionsPanel.setVisible(false);
			playersPanel.setVisible(false);
			actionPanel.setVisible(false);
			navigatorPanel.setVisible(true);
			boardPanel.setVisible(true);
			showDownPanel.setVisible(false);
			flopPanel.setVisible(true);
			turnPanel.setVisible(false);
			riverPanel.setVisible(false);
			optionsButton.setVisible(false);
			newButton.setVisible(true);
			playersButton.setVisible(false);
			flopButton.setVisible(true);
			turnButton.setVisible(false);
			riverButton.setVisible(false);
			showDownButton.setVisible(false);
			postButton.setVisible(false);
			break;

		case VIEW_TURN:
			optionsPanel.setVisible(false);
			playersPanel.setVisible(false);
			actionPanel.setVisible(false);
			navigatorPanel.setVisible(true);
			boardPanel.setVisible(true);
			showDownPanel.setVisible(false);
			flopPanel.setVisible(false);
			turnPanel.setVisible(true);
			riverPanel.setVisible(false);
			optionsButton.setVisible(false);
			newButton.setVisible(true);
			playersButton.setVisible(false);
			flopButton.setVisible(false);
			turnButton.setVisible(true);
			riverButton.setVisible(false);
			showDownButton.setVisible(false);
			postButton.setVisible(false);
			break;

		case VIEW_RIVER:
			optionsPanel.setVisible(false);
			playersPanel.setVisible(false);
			actionPanel.setVisible(false);
			navigatorPanel.setVisible(true);
			boardPanel.setVisible(true);
			showDownPanel.setVisible(false);
			flopPanel.setVisible(false);
			turnPanel.setVisible(false);
			riverPanel.setVisible(true);
			optionsButton.setVisible(false);
			newButton.setVisible(true);
			playersButton.setVisible(false);
			flopButton.setVisible(false);
			turnButton.setVisible(false);
			riverButton.setVisible(true);
			showDownButton.setVisible(false);
			postButton.setVisible(false);
			break;

		case VIEW_SHOWDOWN:
			optionsPanel.setVisible(false);
			playersPanel.setVisible(false);
			actionPanel.setVisible(false);
			navigatorPanel.setVisible(true);
			boardPanel.setVisible(false);
			showDownPanel.setVisible(true);
			flopPanel.setVisible(false);
			turnPanel.setVisible(false);
			riverPanel.setVisible(true);
			optionsButton.setVisible(false);
			newButton.setVisible(true);
			playersButton.setVisible(false);
			flopButton.setVisible(false);
			turnButton.setVisible(false);
			riverButton.setVisible(false);
			showDownButton.setVisible(true);
			postButton.setVisible(false);
			break;

		case VIEW_POST:
			optionsPanel.setVisible(false);
			playersPanel.setVisible(false);
			actionPanel.setVisible(false);
			navigatorPanel.setVisible(true);
			boardPanel.setVisible(false);
			showDownPanel.setVisible(false);
			flopPanel.setVisible(false);
			turnPanel.setVisible(false);
			riverPanel.setVisible(false);
			optionsButton.setVisible(false);
			newButton.setVisible(true);
			playersButton.setVisible(false);
			flopButton.setVisible(false);
			turnButton.setVisible(false);
			riverButton.setVisible(false);
			showDownButton.setVisible(false);
			postButton.setVisible(true);
			break;

		default:
			break;
		}
		pack();
		setLocationRelativeTo(null);
	}

	public void createPlayersPanel(int n, String limit) {
		int[] positions = GameOptions.SH_POSITIONS;
		if (n == GameOptions.FULLRING) {
			positions = GameOptions.FR_POSITIONS;
		}
		playersPanel.removeAll();
		playerActivities = new JCheckBox[n];
		playerNames = new JTextField[n];
		playerStacks = new JTextField[n];
		playerCards = new JTextField[n];
		heroCombo = new JComboBox();
		Box playersBox = Box.createVerticalBox();
		for (int i = 0; i < n; i++) {
			heroCombo.addItem(Player.getPositionName(positions[i]));
			JPanel newPlayerPanel = new JPanel();
			playerActivities[i] = new JCheckBox();
			playerActivities[i].setSelected(true);
			playerNames[i] = new JTextField("Player" + (i + 1), 4);
			newPlayerPanel.add(playerActivities[i]);
			newPlayerPanel.add(new JLabel(Player.getPositionName(positions[i])
					+ ": "));
			newPlayerPanel.add(playerNames[i]);
			playerStacks[i] = new JTextField(limit, 4);
			newPlayerPanel.add(playerStacks[i]);
			playerCards[i] = new JTextField(3);
			newPlayerPanel.add(playerCards[i]);
			playersBox.add(newPlayerPanel);
		}
		playerActivities[n - 1].setEnabled(false);
		playerActivities[n - 2].setEnabled(false);
		JPanel heroPanel = new JPanel();
		heroPanel.add(new JLabel("Hero is: "));
		heroPanel.add(heroCombo);
		playersBox.add(heroPanel);
		playersPanel.add(playersBox);
	}

	public JPanel getLeftPanel() {
		return leftPanel;
	}

	public void setLeftPanel(JPanel leftPanel) {
		this.leftPanel = leftPanel;
	}

	public JPanel getOptionsPanel() {
		return optionsPanel;
	}

	public void setOptionsPanel(JPanel optionsPanel) {
		this.optionsPanel = optionsPanel;
	}

	public JRadioButton getSRadio() {
		return sRadio;
	}

	public void setSRadio(JRadioButton radio) {
		sRadio = radio;
	}

	public JRadioButton getFRadio() {
		return fRadio;
	}

	public void setFRadio(JRadioButton radio) {
		fRadio = radio;
	}

	public JComboBox getLimitsList() {
		return limitsList;
	}

	public void setLimitsList(JComboBox limitsList) {
		this.limitsList = limitsList;
	}

	public JPanel getPlayersPanel() {
		return playersPanel;
	}

	public void setPlayersPanel(JPanel playersPanel) {
		this.playersPanel = playersPanel;
	}

	public JCheckBox[] getPlayerActivities() {
		return playerActivities;
	}

	public void setPlayerActivities(JCheckBox[] playerActivities) {
		this.playerActivities = playerActivities;
	}

	public JTextField[] getPlayerStacks() {
		return playerStacks;
	}

	public void setPlayerStacks(JTextField[] playerStacks) {
		this.playerStacks = playerStacks;
	}

	public JTextField[] getPlayerCards() {
		return playerCards;
	}

	public void setPlayerCards(JTextField[] playerCards) {
		this.playerCards = playerCards;
	}

	public JComboBox getHeroCombo() {
		return heroCombo;
	}

	public void setHeroCombo(JComboBox heroCombo) {
		this.heroCombo = heroCombo;
	}

	public JPanel getWizzardPanel() {
		return wizzardPanel;
	}

	public void setWizzardPanel(JPanel wizzardPanel) {
		this.wizzardPanel = wizzardPanel;
	}

	public JPanel getBoardPanel() {
		return boardPanel;
	}

	public void setBoardPanel(JPanel boardPanel) {
		this.boardPanel = boardPanel;
	}

	public JPanel getFlopPanel() {
		return flopPanel;
	}

	public void setFlopPanel(JPanel flopPanel) {
		this.flopPanel = flopPanel;
	}

	public JPanel getTurnPanel() {
		return turnPanel;
	}

	public void setTurnPanel(JPanel turnPanel) {
		this.turnPanel = turnPanel;
	}

	public JPanel getRiverPanel() {
		return riverPanel;
	}

	public void setRiverPanel(JPanel riverPanel) {
		this.riverPanel = riverPanel;
	}

	public JTextField getFlop() {
		return flop;
	}

	public void setFlop(JTextField flop) {
		this.flop = flop;
	}

	public JTextField getTurn() {
		return turn;
	}

	public void setTurn(JTextField turn) {
		this.turn = turn;
	}

	public JTextField getRiver() {
		return river;
	}

	public void setRiver(JTextField river) {
		this.river = river;
	}

	public ActionPanel getActionPanel() {
		return actionPanel;
	}

	public void setActionPanel(ActionPanel actionPanel) {
		this.actionPanel = actionPanel;
	}

	public ReplayerPanel getReplayerPanel() {
		return replayerPanel;
	}

	public void setReplayerPanel(ReplayerPanel replayerPanel) {
		this.replayerPanel = replayerPanel;
	}

	public JPanel getNavigatorPanel() {
		return navigatorPanel;
	}

	public void setNavigatorPanel(JPanel navigatorPanel) {
		this.navigatorPanel = navigatorPanel;
	}

	public JPanel getHistoryPanel() {
		return historyPanel;
	}

	public void setHistoryPanel(JPanel historyPanel) {
		this.historyPanel = historyPanel;
	}

	public JTextArea getHistory() {
		return history;
	}

	public void setHistory(JTextArea history) {
		this.history = history;
	}

	public JButton getOptionsButton() {
		return optionsButton;
	}

	public void setOptionsButton(JButton optionsButton) {
		this.optionsButton = optionsButton;
	}

	public JButton getNewButton() {
		return newButton;
	}

	public void setNewButton(JButton newButton) {
		this.newButton = newButton;
	}

	public JButton getPlayersButton() {
		return playersButton;
	}

	public void setPlayersButton(JButton playersButton) {
		this.playersButton = playersButton;
	}

	public JButton getFlopButton() {
		return flopButton;
	}

	public void setFlopButton(JButton flopButton) {
		this.flopButton = flopButton;
	}

	public JButton getTurnButton() {
		return turnButton;
	}

	public void setTurnButton(JButton turnButton) {
		this.turnButton = turnButton;
	}

	public JButton getRiverButton() {
		return riverButton;
	}

	public void setRiverButton(JButton riverButton) {
		this.riverButton = riverButton;
	}

	public JPanel getShowDownPanel() {
		return showDownPanel;
	}

	public void setShowDownPanel(JPanel showDownPanel) {
		this.showDownPanel = showDownPanel;
	}

	public JComboBox getShowDownCombo() {
		return showDownCombo;
	}

	public void setShowDownCombo(JComboBox showDownCombo) {
		this.showDownCombo = showDownCombo;
	}

	public JButton getShowDownButton() {
		return showDownButton;
	}

	public void setShowDownButton(JButton showDownButton) {
		this.showDownButton = showDownButton;
	}

	public JTextField[] getPlayerNames() {
		return playerNames;
	}

	public void setPlayerNames(JTextField[] playerNames) {
		this.playerNames = playerNames;
	}

	public JButton getPostButton() {
		return postButton;
	}

	public void setPostButton(JButton postButton) {
		this.postButton = postButton;
	}
}
