package by.zerg.pokerhandsbuilder.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.io.IOException;
import java.util.StringTokenizer;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

import by.zerg.pokerhandsbuilder.domain.Action;
import by.zerg.pokerhandsbuilder.domain.GameOptions;
import by.zerg.pokerhandsbuilder.domain.Player;
import by.zerg.pokerhandsbuilder.replayer.Replayer;

public class ReplayerPanel extends JPanel {
	private Replayer replayer;
	private Image tableImage;
	private Image seatImage;
	private Image seatActionImage;
	private Image deckImage;
	private Image buttonImage;

	public ReplayerPanel() {
		setPreferredSize(new Dimension(839, 570));
		try {
			tableImage = ImageIO.read(getClass()
					.getResource("images/Table.png"));
			seatImage = ImageIO.read(getClass().getResource("images/Seat.png"));
			seatActionImage = ImageIO.read(getClass().getResource(
					"images/SeatAction.png"));
			deckImage = ImageIO.read(getClass().getResource("images/Deck.png"));
			buttonImage = ImageIO.read(getClass().getResource(
					"images/Button.png"));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
		g.drawImage(tableImage, 0, 0, null);
		GameOptions options = replayer.getOptions();
		int n = options.getGameType();
		if (replayer.getState() >= Replayer.INIT_PLAYERS) {
			for (int i = 0; i < n; i++) {
				int x = (int) (346 + 345 * Math.sin(2 * Math.PI * i / n));
				int y = (int) (279 + 200 * Math.cos(2 * Math.PI * i / n));
				g.drawImage(seatImage, x, y, null);
			}
		}
		if (replayer.getState() >= Replayer.PREFLOP_ACTION) {
			Player[] players = options.getPlayers();
			for (int i = 0; i < n; i++) {
				int x = (int) (346 + 345 * Math.sin(-2 * Math.PI * i / n));
				int y = (int) (279 + 200 * Math.cos(-2 * Math.PI * i / n));
				if (players[i].getPosition() == Player.BU) {
					g.drawImage(buttonImage, x, y + 60, null);
				}
				if (players[i] == replayer.getPlayers().get(0)) {
					g.drawImage(seatActionImage, x, y, null);
				}
				g.setColor(Color.WHITE);
				g.setFont(new Font("Dialog", Font.BOLD, 12));
				if (players[i].getAction() != null) {
					g.drawString(Action.getActionName(players[i].getAction()
							.getType()), x + 60, y + 20);
					if (players[i].isActive()
							&& players[i].getAction().getType() != Action.FOLD
							&& players[i].getAction().getType() != Action.CHECK) {
						int x1 = (int) (410 + 235 * Math.sin(-2 * Math.PI * i
								/ n));
						int y1 = (int) (275 + 120 * Math.cos(-2 * Math.PI * i
								/ n));
						g.drawString("$" + players[i].getAction().getAmount(),
								x1, y1);
					}
				} else {
					g.drawString(players[i].getNickName(), x + 60, y + 20);
				}
				if (players[i].isActive()) {
					g.drawString("$" + players[i].getStack(), x + 60, y + 40);
				} else {
					g.drawString("Sitting Out", x + 60, y + 40);
				}
				if (players[i].getCards().trim().equals("")) {
					g.drawImage(deckImage, x + 25, y - 70, x + 75, y, 650, 0,
							700, 70, null);
					g.drawImage(deckImage, x + 75, y - 70, x + 125, y, 650, 0,
							700, 70, null);
				} else {
					String cardName = players[i].getCards().substring(0, 1);
					String suitName = players[i].getCards().substring(1, 2);
					int x1 = 50 * Player.getCardNumber(cardName);
					int y1 = 70 * Player.getCardSuit(suitName);
					g.drawImage(deckImage, x + 25, y - 70, x + 75, y, x1, y1,
							x1 + 50, y1 + 70, null);

					cardName = players[i].getCards().substring(3, 4);
					suitName = players[i].getCards().substring(4, 5);
					x1 = 50 * Player.getCardNumber(cardName);
					y1 = 70 * Player.getCardSuit(suitName);
					g.drawImage(deckImage, x + 75, y - 70, x + 125, y, x1, y1,
							x1 + 50, y1 + 70, null);
				}
			}
			String pot = "Pot = $" + replayer.getPot();
			g.drawString(pot, 400, 310);
		}
		if (replayer.getState() >= Replayer.FLOP_ACTION) {
			int x = 295;
			int y = 220;
			String flop = replayer.getFlop();
			StringTokenizer st = new StringTokenizer(flop, " ,");
			for (int i = 0; i < 3; i++) {
				String card = " s";
				if (st.hasMoreTokens()) {
					card = st.nextToken();
				}
				String cardName = card.substring(0, 1);
				String suitName = card.substring(1, 2);
				int x1 = 50 * Player.getCardNumber(cardName);
				int y1 = 70 * Player.getCardSuit(suitName);
				g.drawImage(deckImage, x, y, x + 50, y + 70, x1, y1, x1 + 50,
						y1 + 70, null);
				x += 50;
			}
		}
		if (replayer.getState() >= Replayer.TURN_ACTION) {
			int x = 445;
			int y = 220;
			String card = replayer.getTurn().trim();
			if (card == null || card.equals("")) {
				card = " s";
			}
			String cardName = card.substring(0, 1);
			String suitName = card.substring(1, 2);
			int x1 = 50 * Player.getCardNumber(cardName);
			int y1 = 70 * Player.getCardSuit(suitName);
			g.drawImage(deckImage, x, y, x + 50, y + 70, x1, y1, x1 + 50,
					y1 + 70, null);
		}
		if (replayer.getState() >= Replayer.RIVER_ACTION) {
			int x = 495;
			int y = 220;
			String card = replayer.getRiver().trim();
			if (card == null || card.equals("")) {
				card = " s";
			}
			String cardName = card.substring(0, 1);
			String suitName = card.substring(1, 2);
			int x1 = 50 * Player.getCardNumber(cardName);
			int y1 = 70 * Player.getCardSuit(suitName);
			g.drawImage(deckImage, x, y, x + 50, y + 70, x1, y1, x1 + 50,
					y1 + 70, null);
		}
		setVisible(false);
		setVisible(true);
	}

	public Replayer getReplayer() {
		return replayer;
	}

	public void setReplayer(Replayer replayer) {
		this.replayer = replayer;
	}

	public Image getTableImage() {
		return tableImage;
	}

	public void setTableImage(Image tableImage) {
		this.tableImage = tableImage;
	}

	public Image getSeatImage() {
		return seatImage;
	}

	public void setSeatImage(Image seatImage) {
		this.seatImage = seatImage;
	}

	public Image getDeckImage() {
		return deckImage;
	}

	public void setDeckImage(Image deckImage) {
		this.deckImage = deckImage;
	}
}
