package by.zerg.pokerhandsbuilder.replayer;

import java.awt.Desktop;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import by.zerg.pokerhandsbuilder.domain.Action;
import by.zerg.pokerhandsbuilder.domain.GameOptions;
import by.zerg.pokerhandsbuilder.domain.Player;
import by.zerg.pokerhandsbuilder.gui.ActionPanel;
import by.zerg.pokerhandsbuilder.gui.MainFrame;

public class Replayer implements ActionListener {
	public static final int INIT_GAME = 0;
	public static final int INIT_PLAYERS = 1;
	public static final int PREFLOP_ACTION = 2;
	public static final int FLOP_ACTION = 3;
	public static final int TURN_ACTION = 4;
	public static final int RIVER_ACTION = 5;
	public static final int SHOWDOWN = 6;
	public static final int POST = 7;

	private int state;
	private List<Player> players;
	private float pot;
	private float rake;
	private float returnedAmount;
	private String flop;
	private String turn;
	private String river;
	private Replayer prev;
	private Replayer next;
	private String history;
	private float lastRaise;

	private MainFrame gui;
	private GameOptions options;

	public Replayer() {
		options = new GameOptions();
		gui = new MainFrame(this);
		gui.getReplayerPanel().setReplayer(this);
		gui.switchView(MainFrame.VIEW_OPTIONS);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String actionCommand = e.getActionCommand();
		if (MainFrame.OPTIONS_SELECTED.equals(actionCommand)) {
			int gameType = GameOptions.SHORTHANDED;
			if (gui.getFRadio().isSelected()) {
				gameType = GameOptions.FULLRING;
			}
			String limit = ((String) gui.getLimitsList().getSelectedItem())
					.substring(2);
			Float bigBlind = Float.valueOf(limit) / 100;
			options.setGameType(gameType);
			options.setLimit(limit);
			options.setBigBlind(bigBlind);
			options.setSmallBlind(bigBlind / 2);
			history = "***** Hand History for Game 70530031803 ***** (Poker Stars)\n$"
					+ limit
					+ " USD NL Texas Hold'em - Monday, November 14, 09:31:07 ET 2011\nTable Paulina II (Real Money)\nSeat "
					+ (gameType - 2) + " is the button";
			gui.createPlayersPanel(gameType, limit);
			gui.switchView(MainFrame.VIEW_PLAYERS);
			state = INIT_PLAYERS;
		} else if (MainFrame.SET_OPTIONS.equals(actionCommand)) {
			options = new GameOptions();
			history = null;
			pot = 0;
			rake = 0;
			returnedAmount = 0;
			lastRaise = 0;
			flop = null;
			turn = null;
			river = null;
			gui.switchView(MainFrame.VIEW_OPTIONS);
			state = INIT_GAME;
		} else if (MainFrame.PLAYERS_SELECTED.equals(actionCommand)) {
			int n = options.getGameType();
			Player[] players = new Player[n];
			this.players = new ArrayList<Player>();
			int[] positions = GameOptions.SH_POSITIONS;
			if (n == GameOptions.FULLRING) {
				positions = GameOptions.FR_POSITIONS;
			}
			for (int i = 0; i < n; i++) {
				players[i] = new Player();
				if (Player.getPositionName(positions[i]).equals(
						gui.getHeroCombo().getSelectedItem())) {
					players[i].setNickName("Hero");
					options.setHero(players[i]);
				} else {
					players[i].setNickName(gui.getPlayerNames()[i].getText());
				}
				players[i].setActive(gui.getPlayerActivities()[i].isSelected());
				players[i].setPosition(positions[i]);
				players[i].setStack(Float.valueOf(gui.getPlayerStacks()[i]
						.getText()));
				players[i].setCards(gui.getPlayerCards()[i].getText());
				if (players[i].isActive()) {
					this.players.add(players[i]);
				}
				history += "\nSeat " + (i + 1) + ": "
						+ players[i].getNickName() + " ( $"
						+ players[i].getStack() + " USD )";
			}
			options.setPlayers(players);

			Player player = players[n - 2];
			float amount = options.getSmallBlind();
			player.postSmallBlind(amount);
			pot += amount;
			history += "\n" + player.getNickName() + " posts small blind [$"
					+ amount + " USD].";

			returnedAmount = amount;
			player = players[n - 1];
			amount = options.getBigBlind();
			player.postBigBlind(amount);
			pot += amount;
			lastRaise = amount;
			history += "\n" + player.getNickName() + " posts big blind [$"
					+ amount + " USD].\n** Dealing down cards **\nDealt to "
					+ options.getHero().getNickName() + " [  "
					+ options.getHero().getCards() + " ]";
			gui.getActionPanel().switchView(ActionPanel.VIEW1);
			gui.switchView(MainFrame.VIEW_ACTION);
			state = PREFLOP_ACTION;
		} else if (MainFrame.FLOP_SELECTED.equals(actionCommand)) {
			flop = gui.getFlop().getText();
			history += "\n** Dealing Flop ** [ " + flop + " ]";
			for (Player player : players) {
				if (player.getAction().getType() != Action.ALLIN) {
					player.setAction(null);
				}
			}
			gui.switchView(MainFrame.VIEW_ACTION);
			state = FLOP_ACTION;
			lastRaise = 0;
			returnedAmount = 0;
			resetPlayers();
		} else if (MainFrame.TURN_SELECTED.equals(actionCommand)) {
			turn = gui.getTurn().getText();
			history += "\n** Dealing Turn ** [ " + turn + " ]";
			for (Player player : players) {
				if (player.getAction().getType() != Action.ALLIN) {
					player.setAction(null);
				}
			}
			gui.switchView(MainFrame.VIEW_ACTION);
			state = TURN_ACTION;
			lastRaise = 0;
			returnedAmount = 0;
			resetPlayers();
		} else if (MainFrame.RIVER_SELECTED.equals(actionCommand)) {
			river = gui.getRiver().getText();
			history += "\n** Dealing River ** [ " + river + " ]";
			for (Player player : players) {
				if (player.getAction().getType() != Action.ALLIN) {
					player.setAction(null);
				}
			}
			gui.switchView(MainFrame.VIEW_ACTION);
			state = RIVER_ACTION;
			lastRaise = 0;
			returnedAmount = 0;
			resetPlayers();
		} else if (MainFrame.SHOWDOWN_SELECTED.equals(actionCommand)) {
			for (Player player : players) {
				history += "\n" + player.getNickName() + " shows ["
						+ player.getCards() + " ]";
			}
			String winner = (String) gui.getShowDownCombo().getSelectedItem();
			history += "\n" + winner + "  wins $" + pot + " USD from main pot";
			gui.switchView(MainFrame.VIEW_POST);
			state = SHOWDOWN;
		} else if (MainFrame.POST_SELECTED.equals(actionCommand)) {
			/* new code 
			try {
				String charset = "UTF-8";
				String query = String.format("hh=%s",
						URLEncoder.encode(history, charset));

				URLConnection connection = new URL("http://weaktight.com/c/")
						.openConnection();
				connection.setDoOutput(true); // Triggers POST.
				connection.setRequestProperty("Accept-Charset", charset);
				connection.setRequestProperty("Content-Type",
						"application/x-www-form-urlencoded;charset=" + charset);
				OutputStream output = null;
				try {
					output = connection.getOutputStream();
					output.write(query.getBytes(charset));
				} finally {
					if (output != null)
						try {
							output.close();
						} catch (IOException logOrIgnore) {
						}
				}
				InputStream response = connection.getInputStream();
				// ...
				int ch = -1;
				do {
					ch = response.read();
					System.out.print((char) ch);
				} while (ch != -1);
			} catch (Throwable t) {
				t.printStackTrace();
			}
			*/

			/* old code */
			Clipboard clipboard = Toolkit.getDefaultToolkit()
					.getSystemClipboard();
			clipboard.setContents(new StringSelection(history), null);
			try {
				Desktop.getDesktop().browse(
						new URI("http://weaktight.com/hand/"));
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (URISyntaxException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

			gui.switchView(MainFrame.VIEW_OPTIONS);
			state = -1;
		} else if (ActionPanel.FOLD.equals(actionCommand)) {
			Player player = players.get(0);
			player.fold();
			history += "\n" + player.getNickName() + " folds";
			players.remove(0);
		} else if (ActionPanel.CHECK.equals(actionCommand)) {
			Player player = players.get(0);
			history += "\n" + player.getNickName() + " checks";
			player.check();
			players.add(players.remove(0));
		} else if (ActionPanel.CALL.equals(actionCommand)) {
			Player player = players.get(0);
			float toCall = player.call(lastRaise);
			pot += toCall;
			returnedAmount = lastRaise - toCall;
			history += "\n" + player.getNickName() + " calls [$" + toCall
					+ " USD]";
			players.add(players.remove(0));
			if (state == PREFLOP_ACTION && lastRaise == options.getBigBlind()
					&& players.get(0).getPosition() == Player.BB) {
				gui.getActionPanel().switchView(ActionPanel.VIEW2);
			}
		} else if (ActionPanel.BET.equals(actionCommand)) {
			Player player = players.get(0);
			float amount = Float.valueOf(gui.getActionPanel().getActionAmount()
					.getText());
			amount = player.bet(amount);
			pot += amount;
			returnedAmount = amount;
			lastRaise = amount;
			history += "\n" + player.getNickName() + " bets [$" + amount
					+ " USD]";
			players.add(players.remove(0));
			gui.getActionPanel().switchView(ActionPanel.VIEW1);
		} else if (ActionPanel.RAISE.equals(actionCommand)) {
			Player player = players.get(0);
			float amount = Float.valueOf(gui.getActionPanel().getActionAmount()
					.getText());
			amount = player.raise(amount);
			pot += amount;
			returnedAmount = amount - lastRaise;
			lastRaise = amount;
			history += "\n" + player.getNickName() + " raises [$" + amount
					+ " USD]";
			players.add(players.remove(0));
		} else { // TODO Temporary navigator stub
			return;
		}
		if (state >= PREFLOP_ACTION && players.size() == 1) {
			if (returnedAmount > 0) {
				history += "\n" + players.get(0).getNickName() + " wins $"
						+ returnedAmount + " USD";
			}
			history += "\n" + players.get(0).getNickName() + " wins $"
					+ (pot - returnedAmount) + " USD from main pot";
			gui.switchView(MainFrame.VIEW_POST);
		} else if (isActionClosed() || isAllIn()) {
			switch (state) {
			case PREFLOP_ACTION:
				gui.getActionPanel().switchView(ActionPanel.VIEW3);
				gui.switchView(MainFrame.VIEW_FLOP);
				break;

			case FLOP_ACTION:
				gui.getActionPanel().switchView(ActionPanel.VIEW3);
				gui.switchView(MainFrame.VIEW_TURN);
				break;

			case TURN_ACTION:
				gui.getActionPanel().switchView(ActionPanel.VIEW3);
				gui.switchView(MainFrame.VIEW_RIVER);
				break;

			case RIVER_ACTION:
				gui.getShowDownCombo().removeAllItems(); // Fixes Showdown
				// ComboBox bug
				for (Player player : players) {
					gui.getShowDownCombo().addItem(player.getNickName());
				}
				gui.switchView(MainFrame.VIEW_SHOWDOWN);
				break;
			default:
				break;
			}

		}
		gui.getHistory().setText(history);
		gui.getActionPanel().getActionAmount().setText(null);
	}

	public void resetPlayers() {
		Collections.sort(players, new Comparator<Player>() {
			@Override
			public int compare(Player p1, Player p2) {
				return p1.getPosition() - p2.getPosition();
			}

		});
	}

	public boolean isAllIn() {
		if (players == null) {
			return false;
		}
		int aN = 0;
		for (Player player : players) {
			if (player.getAction() == null) {
				return false;
			} else if (player.getAction().getType() == Action.ALLIN) {
				aN++;
			}
		}
		if (aN == players.size()) {
			return true;
		}
		return false;
	}

	public boolean isActionClosed() {
		if (players == null) {
			return false;
		}
		if (state == PREFLOP_ACTION
				&& players.get(players.size() - 1).getAction().getType() == Action.CHECK) {
			return true;
		}
		boolean flag = true;
		for (Player player : players) {
			if (player.getAction() == null
					|| player.getAction().getType() == Action.POST_SB
					|| player.getAction().getType() == Action.POST_BB) {
				return false;
			}
			if (player.getAction().getAmount() != lastRaise) {
				flag = false;
				break;
			}
		}
		return flag;
	}

	public int getState() {
		return state;
	}

	public void setState(int state) {
		this.state = state;
	}

	public List<Player> getPlayers() {
		return players;
	}

	public void setPlayers(List<Player> players) {
		this.players = players;
	}

	public float getPot() {
		return pot;
	}

	public void setPot(float pot) {
		this.pot = pot;
	}

	public float getRake() {
		return rake;
	}

	public void setRake(float rake) {
		this.rake = rake;
	}

	public String getFlop() {
		return flop;
	}

	public void setFlop(String flop) {
		this.flop = flop;
	}

	public String getTurn() {
		return turn;
	}

	public void setTurn(String turn) {
		this.turn = turn;
	}

	public String getRiver() {
		return river;
	}

	public void setRiver(String river) {
		this.river = river;
	}

	public Replayer getPrev() {
		return prev;
	}

	public void setPrev(Replayer prev) {
		this.prev = prev;
	}

	public Replayer getNext() {
		return next;
	}

	public void setNext(Replayer next) {
		this.next = next;
	}

	public String getHistory() {
		return history;
	}

	public void setHistory(String history) {
		this.history = history;
	}

	public GameOptions getOptions() {
		return options;
	}

	public void setOptions(GameOptions options) {
		this.options = options;
	}

	public MainFrame getGui() {
		return gui;
	}

	public void setGui(MainFrame gui) {
		this.gui = gui;
	}

	public static void main(String[] args) {
		new Replayer();
	}
}
